//=========================================================================
//
//	FILE:				CP_DemoShapesView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View to display shapes using CP_Graphics.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 3 $
//						Latest Checkout:  $Date: 2004-12-21 19:27:54 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================

	// Precompile header support
#include "CP_Precomp.h"

	// CP_Graphics
#include "CP_DemoShapesView.h"

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_Errors.h"
#include "CP_Exceptions.h"
#include "CP_Graphics.h"
#include "CP_Types.h"

#if TARGET_CC_METROWERKS
#pragma mark === Construction / Destruction ===
#endif

//=========================================================================
//	FUNCTION:	CP_DemoShapesView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The CP_View that contains this view.
*	@param		inUpperLeft			Upperleft coordinate of view.
*	@param		inContentSize		The size of the view.
*	@param		inUseOffscreen		Flag to indicate if the image should be
*									drawn using an offscreen canvas.
*	
*	@exception	none
*/
CP_DemoShapesView::CP_DemoShapesView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize )
		: CPLAT::CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
}

//=========================================================================
//	FUNCTION:	~CP_DemoShapesView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_DemoShapesView::~CP_DemoShapesView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark === Drawing ===
#endif

static inline double radians(double degrees)		{ return degrees * CP_PI / 180; }

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Perform the actual drawing of the view.
*
*	@param		inCanvas			CP_Canvas object to draw into.
*	@param		inUpdateArea		CP_Region containing the area to draw.
*	
*	@exception	none
*
*	@return		void
*/
void CP_DemoShapesView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Graphics			graphicsOne( inCanvas );
	SInt16						verPosn = 5;

	verPosn += DrawRects( graphicsOne, verPosn );
	verPosn += DrawBezier( graphicsOne, verPosn );
	verPosn += DrawPolygons( graphicsOne, verPosn );
	
		// draw a face
	#define	kDegree			( CP_PI / 180 )

		// face
	CPLAT::CP_ARGBColor		fillColor( 1.0, 0.83, 0.0, 1.0 );
	CPLAT::CP_ARGBColor		strokeColor( 0.0, 0.0, 0.0, 1.0 );
	CPLAT::CP_Rect			faceBounds( 0, verPosn, 100, verPosn + 100 );
	CPLAT::CP_Rect			leftEyeBounds( 25, verPosn + 20, 45, verPosn + 50 );
	CPLAT::CP_Rect			rightEyeBounds( 55, verPosn + 20, 75, verPosn + 50 );
	
		// outter circle
	graphicsOne.SetFillColor( fillColor );
	graphicsOne.SetStrokeColor( strokeColor );
	graphicsOne.FillOval( faceBounds );
	graphicsOne.StrokeOval( faceBounds );
	
		// eyes
	fillColor.Set( 0.0, 0.0, 0.0, 1.0 );
	graphicsOne.SetAlpha( 1.0 );
	graphicsOne.SetFillColor( fillColor );
	graphicsOne.FillOval( leftEyeBounds );
	graphicsOne.FillOval( rightEyeBounds );
	
		// mouth
//	graphicsOne.BeginPath();
//	graphicsOne.PathAddArc( 50, verPosn + 50, 40, 185 * kDegree, 355 * kDegree, true );
//	graphicsOne.StrokePath();

		// nose
	CPLAT::CP_Point_Array			points;
	
	points.push_back( CPLAT::CP_Point( 50, verPosn + 55 ) );
	points.push_back( CPLAT::CP_Point( 40, verPosn + 65 ) );
	points.push_back( CPLAT::CP_Point( 60, verPosn + 65 ) );
	points.push_back( CPLAT::CP_Point( 50, verPosn + 55 ) );
	
	graphicsOne.FillPoly( points );
}

//=========================================================================
//	FUNCTION:	DrawRects
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Draw example rects using CP_Graphics. Called from the OnDraw() method.
*
*	@param		inGraphics			The CP_Graphics object we are drawing to.
*	@param		inStartVerPosn		The starting vertical position for drawing.
*	
*	@exception	none
*
*	@return		SInt16				The height in pixel occupied by this method
*									to draw its shpaes.
*/
SInt16 CP_DemoShapesView::DrawRects( CPLAT::CP_Graphics& inGraphics, SInt16 inStartVerPosn )
{
	CPLAT::CP_Rect		shapeBounds;
	CPLAT::CP_ARGBColor	strokeColor( 1.0, 0.0, 0.0, 1.0 );
	CPLAT::CP_ARGBColor	fillColor( 0.0, 1.0, 0.0, 1.0 );
	SInt16				verPosn = inStartVerPosn;
	
		// solid rects
		// red rect, red frame / green interior rect / solid blue rect
	shapeBounds.Set( 5, verPosn, 35, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeRect( shapeBounds );
		
	shapeBounds.Set( 40, verPosn, 70, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );
	inGraphics.StrokeRect( shapeBounds );

	shapeBounds.Set( 75, verPosn, 105, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );
	
		// solid rects over solid for alpha blending
	shapeBounds.Set( 130, verPosn - 5, 240, verPosn + 40 );
	fillColor.Set( 0.6, 0.9, 0.95, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );

	inGraphics.SetAlpha( 0.2 );
	strokeColor.Set( 1.0, 0.0, 0.0, 1.0 );
	fillColor.Set( 0.0, 1.0, 0.0, 1.0 );
	
	shapeBounds.Set( 135, verPosn, 165, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeRect( shapeBounds );
		
	shapeBounds.Set( 170, verPosn, 200, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );
	inGraphics.StrokeRect( shapeBounds );

	shapeBounds.Set( 205, verPosn, 235, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );

	inGraphics.SetAlpha( 1.0 );

		// round rects
	verPosn += 45;
	
		// solid rects
		// red rect, red frame / green interior rect / solid blue rect
	strokeColor.Set( 1.0, 0.0, 0.0, 1.0 );
	inGraphics.SetStrokeColor( strokeColor );
	fillColor.Set( 0.0, 1.0, 0.0, 1.0 );
	inGraphics.SetStrokeColor( fillColor );

	shapeBounds.Set( 5, verPosn, 35, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeRoundRect( shapeBounds, 12, 12 );
		
	shapeBounds.Set( 40, verPosn, 70, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRoundRect( shapeBounds, 12, 12 );
	inGraphics.StrokeRoundRect( shapeBounds, 12, 12 );

	shapeBounds.Set( 75, verPosn, 105, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRoundRect( shapeBounds, 12, 12 );
	
		// solid rects over solid for alpha blending
	shapeBounds.Set( 130, verPosn - 5, 240, verPosn + 40 );
	fillColor.Set( 0.6, 0.9, 0.95, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );

	inGraphics.SetAlpha( 0.2 );
	strokeColor.Set( 1.0, 0.0, 0.0, 1.0 );
	fillColor.Set( 0.0, 1.0, 0.0, 1.0 );
	
	shapeBounds.Set( 135, verPosn, 165, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeRoundRect( shapeBounds, 12, 12 );
		
	shapeBounds.Set( 170, verPosn, 200, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRoundRect( shapeBounds, 12, 12 );
	inGraphics.StrokeRoundRect( shapeBounds, 12, 12 );

	shapeBounds.Set( 205, verPosn, 235, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRoundRect( shapeBounds, 12, 12 );

	inGraphics.SetAlpha( 1.0 );

		// round rects
	verPosn += 45;
	
		// ovals
		// red rect, red frame / green interior rect / solid blue rect
	strokeColor.Set( 1.0, 0.0, 0.0, 1.0 );
	inGraphics.SetStrokeColor( strokeColor );
	fillColor.Set( 0.0, 1.0, 0.0, 1.0 );
	inGraphics.SetStrokeColor( fillColor );

	shapeBounds.Set( 5, verPosn, 35, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeOval( shapeBounds );
		
	shapeBounds.Set( 40, verPosn, 70, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillOval( shapeBounds );
	inGraphics.StrokeOval( shapeBounds );

	shapeBounds.Set( 75, verPosn, 105, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillOval( shapeBounds );
	
		// solid rects over solid for alpha blending
	shapeBounds.Set( 130, verPosn - 5, 240, verPosn + 40 );
	fillColor.Set( 0.6, 0.9, 0.95, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillRect( shapeBounds );

	inGraphics.SetAlpha( 0.2 );
	strokeColor.Set( 1.0, 0.0, 0.0, 1.0 );
	fillColor.Set( 0.0, 1.0, 0.0, 1.0 );
	
	shapeBounds.Set( 135, verPosn, 165, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.StrokeOval( shapeBounds );
		
	shapeBounds.Set( 170, verPosn, 200, verPosn + 30 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillOval( shapeBounds );
	inGraphics.StrokeOval( shapeBounds );

	shapeBounds.Set( 205, verPosn, 235, verPosn + 30 );
	fillColor.Set( 0.0, 0.0, 1.0, 1.0 );
	inGraphics.SetFillColor( fillColor );
	inGraphics.FillOval( shapeBounds );

		// restore to black
	strokeColor.Set( 0.0, 0.0, 0.0, 1.0 );
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetAlpha( 1.0 );
	
	return( verPosn + 25 );		// return the height of the area we occupied plus air space for the start of the next demo section
}

//=========================================================================
//	FUNCTION:	DrawBezier
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Draw example bezier curves using CP_Graphics. Called from the OnDraw() method.
*
*	@param		inGraphics			The CP_Graphics object we are drawing to.
*	@param		inStartVerPosn		The starting vertical position for drawing.
*	
*	@exception	none
*
*	@return		SInt16				The height in pixel occupied by this method
*									to draw its shpaes.
*/
SInt16 CP_DemoShapesView::DrawBezier( CPLAT::CP_Graphics& inGraphics, SInt16 inStartVerPosn )
{
	CPLAT::CP_Point			startPointOne( 10, inStartVerPosn + 100 );
	CPLAT::CP_Point			controlPointOne( 75, inStartVerPosn + 10 );
	CPLAT::CP_Point			controlPointTwo( 80, inStartVerPosn + 50 );
	CPLAT::CP_Point			endPointOne( 100, inStartVerPosn + 150 );
	CPLAT::CP_Point			startPointTwo( 100, inStartVerPosn + 150 );
	CPLAT::CP_Point			controlPointThree( 125, inStartVerPosn + 80 );
	CPLAT::CP_Point			controlPointFour( 175, inStartVerPosn + 200 );
	CPLAT::CP_Point			endPointTwo( 200, inStartVerPosn + 80 );
	SInt16					horOffset = 200;
	CPLAT::CP_ARGBColor		strokeColor( 0.0, 0.0, 0.0, 1.0 );
	
	inGraphics.SetStrokeColor( strokeColor );
	inGraphics.SetAlpha( 1.0 );
	
	inGraphics.StrokeBezier( startPointOne, controlPointOne, controlPointTwo, endPointOne );
	inGraphics.StrokeBezier( startPointTwo, controlPointThree, controlPointFour, endPointTwo );
	
	inGraphics.SetAlpha( 0.3 );
	
	startPointOne.Set( horOffset + 10, inStartVerPosn + 100 );
	controlPointOne.Set( horOffset + 75, inStartVerPosn + 10 );
	controlPointTwo.Set( horOffset + 80, inStartVerPosn + 50 );
	endPointOne.Set( horOffset + 100, inStartVerPosn + 150 );
	startPointTwo.Set( horOffset + 100, inStartVerPosn + 150 );
	controlPointThree.Set( horOffset + 125, inStartVerPosn + 80 );
	controlPointFour.Set( horOffset + 175, inStartVerPosn + 200 );
	endPointTwo.Set( horOffset + 200, inStartVerPosn + 80 );

	inGraphics.StrokeBezier( startPointOne, controlPointOne, controlPointTwo, endPointOne );
	inGraphics.StrokeBezier( startPointTwo, controlPointThree, controlPointFour, endPointTwo );

	return( 160 );
}

//=========================================================================
//	FUNCTION:	DrawPolygons
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Draw example polygons using CP_Graphics. Called from the OnDraw() method.
*
*	@param		inGraphics			The CP_Graphics object we are drawing to.
*	@param		inStartVerPosn		The starting vertical position for drawing.
*	
*	@exception	none
*
*	@return		SInt16				The height in pixel occupied by this method
*									to draw its shpaes.
*/
SInt16 CP_DemoShapesView::DrawPolygons( CPLAT::CP_Graphics& inGraphics, SInt16 inStartVerPosn )
{
	CPLAT::CP_Point_Array			points;
	
	points.push_back( CPLAT::CP_Point( 10, inStartVerPosn ) );
	points.push_back( CPLAT::CP_Point( 50, inStartVerPosn + 50 ) );
	points.push_back( CPLAT::CP_Point( 10, inStartVerPosn + 60 ) );
	points.push_back( CPLAT::CP_Point( 10, inStartVerPosn ) );			// close shape
	
	inGraphics.StrokePoly( points );
	
	return( 60 );
}

#if TARGET_OS_MAC

#pragma mark -
#pragma mark === Mac specific ===
#pragma mark -

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#pragma mark -
#pragma mark === Win specific ===
#pragma mark -

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#pragma mark -
#pragma mark === Linux specific ===
#pragma mark -

#endif	// TARGET_OS_LINUX
