//=========================================================================
//
//	FILE:				CP_Document_App.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_Document demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:30 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// CP_Document
#include "CP_Document_App.h"
#include "CP_Document_Doc.h"

	// CPLAT_Framework
#include "CP_Application.tmpl.h"
#include "CP_Data_ResourceTypes.h"
#include "CP_DocumentFactoryManager.h"
#include "CP_DocumentPolicy_Multi.h"
#include "CP_FontMenu.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_Document_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_Document_Application::CP_Document_Application()
{
}

//=========================================================================
//	FUNCTION:	~CP_Document_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_Document_Application::~CP_Document_Application() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String CP_Document_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to demonstrate basic document handling.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Documents ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	RegisterDocuments
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Register the objects of type CP_Document_Base and their default file type
*	that are used to create a new document when the application requests a new
*	document of a specific type be created. In this demo, we register a document
*	of type 'TEXT'.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Document_Application::RegisterDocuments()
{
	#if TARGET_OS_WIN32
		CPLAT::CP_DocumentRegistration<CP_Document_Doc>::CP_DocumentRegistration( CPLAT::kCP_Text_FileType );
	#else
		CPLAT::CP_DocumentRegistration<CP_Document_Doc>::CP_DocumentRegistration( 'TEXT'/*kCP_Text_FileType*/ );
	#endif
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
