//=========================================================================
//
//	FILE:				CP_Alerts.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A collection of of routines to handle display of alerts
//						and message boxes.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				All routines in the file are in the namespace:
//						CP_Alerts
//
//	VERSION:			Current Revision: $Rev: 46 $
//						Latest Checkout:  $Date: 2005-01-04 14:55:52 -0800 (Tue, 04 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_Alerts
#define _H_CP_Alerts
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Alerts - [namespace]
*	@brief		A collection of classes used to display alerts to the user.
*
*	@ingroup	Alerts
*/
namespace CP_Alerts
{
	/*!
	*	@class		CP_OSAlert
	*	@brief		The alerts in this class use the native OS alert handling APIs
	*				to display the alerts, and not CPLAT UI elements.
	*
	*	@code
	*		void DisplayMessage()
	*		{
	*			CP_String		message( CP_T("My message goes here") );
	*			CP_String		description( CP_T("This is where I would display a description to the user.") );
	*			
	*			CP_Alerts::CP_OSAlert::ShowAlert( message, inDescription );
	*		}
	*	@endcode
	*
	*	@ingroup	Alerts
	*/
	class CP_OSAlert
	{
	public:
		enum
		{
			kCP_AlertType_StopAlert		= 0,		/*!< Stop icon is displayed for this alert. */
			kCP_AlertType_NoteAlert		= 1,		/*!< Note icon is displayed for this alert. */
			kCP_AlertType_CautionAlert	= 2,		/*!< Caution icon is displayed for this alert. */
			kCP_AlertType_PlainAlert	= 3			/*!< No icon is displayed for this alert. */
		};
		
		static void						ShowAlert(
												const CP_String& inMessage,
												const CP_String& inDescription );
	
		static SInt16					ShowAlert(
												const CP_String& inMessage,
												SInt16 inAlertType,
												bool inShowCancel );

	//========================================================================
	// Platform specific routines
	//========================================================================
	#if TARGET_OS_MAC
	private:
		static void						ShowAlert_Imp(
												const CP_String& inMessage,
												const CP_String& inDescription );
		static SInt16					ShowAlert_Imp(
												const CP_String& inMessage,
												SInt16 inAlertType,
												bool inShowCancel );
	#endif

	#if TARGET_OS_WIN32
	private:
		static void						ShowAlert_Imp(
												const CP_String& inMessage,
												const CP_String& inDescription );
		static SInt16					ShowAlert_Imp(
												const CP_String& inMessage,
												SInt16 inAlertType,
												bool inShowCancel );
	#endif

	#if TARGET_OS_LINUX
	private:
		static void						ShowAlert_Imp(
												const CP_String& inMessage,
												const CP_String& inDescription );
		static SInt16					ShowAlert_Imp(
												const CP_String& inMessage,
												SInt16 inAlertType,
												bool inShowCancel );
		static GtkWidget				*AddWidget(
												GtkWidget *widget,
												GtkWidget *packingbox );
	#endif
	};
	
}	// namespace CP_Alerts

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Alerts

