//=========================================================================
//
//	FILE:				CP_CodeFragment.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class to provide code fragment support for CPLAT. A
//						code fragment can be stored in a MacOS shared lib, or a
//						Windows dll, loaded and executed.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 196 $
//						Latest Checkout:  $Date: 2005-05-31 16:14:17 -0700 (Tue, 31 May 2005) $
//
//=========================================================================
#ifndef _H_CP_CodeFragment
#define _H_CP_CodeFragment
#ifndef __BUILD_LINUX__
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
	typedef CFragConnectionID		CP_OSCodeFragment;

	enum CP_E_FragOptions
	{
		kUseExistingFragment,
		kLoadNewFragment
	};
#endif

#if TARGET_OS_WIN32
	typedef HINSTANCE				CP_OSCodeFragment;

	enum CP_E_FragOptions
	{
		kUseExistingFragment
	};
#endif
	
#if TARGET_OS_LINUX
	typedef char *				CP_OSCodeFragment;

	enum CP_E_FragOptions
	{
		kUseExistingFragment
	};
#endif

/*!
*	@class		CP_CodeFragment
*	@brief		Base class to provide code fragment support for CPLAT. A code fragment can be
*				stored in a MacOS shared lib, or a Windows dll, loaded and executed.
*/
class CP_CodeFragment
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.	
										CP_CodeFragment(
												const CP_String& inFragmentName,
												CP_E_FragOptions inOptions = kUseExistingFragment );
										CP_CodeFragment(
												const CP_FileSystemSpec& inFragmentSpec,
												CP_E_FragOptions inOptions = kUseExistingFragment );
	//! Destructor.
										~CP_CodeFragment();
	//@}
										
	//! @name Getters
	//@{
										/*!
										*	Returns the name of the code fragment.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			The name of the code fragment.
										*/
			CP_String					GetName() const													{ return( fFragmentName ); }

										/*!
										*	Returns the CP_OSCodeFragment which is the underlying OS code
										*	fragment.
										*	
										*	@exception	none
										*	
										*	@return		CP_OSCodeFragment	The underlying OS code fragment.
										*/
			CP_OSCodeFragment 			GetOSFragment() const											{ return( fFragment ); }

			void						*GetFunction( const CP_String& inFunctionName ) const;
	//@}

protected:
	
	// Class data
protected:
	CP_String							fFragmentName;		/*! Name of code fragment we want. */
	CP_OSCodeFragment					fFragment;			/*! The OS code fragment. */

	#if TARGET_OS_MAC
		Ptr								fMainAdr;			/*! Ptr to the code fragment. */
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.	
										CP_CodeFragment(
												const CP_FileSystemSpec& inFragmentSpec,
												UInt32 inOffset,
												UInt32 inBytes,
												CP_E_FragOptions inOptions = kUseExistingFragment );
										CP_CodeFragment(
												const CP_String& inFragmentName,
												const void *inAddress,
												UInt32 inBytes,
												CP_E_FragOptions inOptions = kUseExistingFragment );
	//@}

	//! @name Getters
	//@{
	#if TARGET_API_MAC_CARBON
		static void						*GetCGFunction( CFStringRef inFunctionName );
		static void 					*GetCarbonFunction( CFStringRef inFunctionName );
		static void 					*GetHIObjectFunction( CFStringRef inFunctionName );
	#endif
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_CodeFragment

