//=========================================================================
//
//	FILE:				CP_DataSource.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Virtual base class to handle encoding of data.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_DataEncoder
#define _H_CP_DataEncoder
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_HandleBlock.h"
#include "CP_Object.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class		CP_DataEncoder
*	\brief		Virtual base class to handle encoding of data.
*/
class CP_EXPORT CP_DataEncoder : public CP_Object
{
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataEncoder();
	//! Destructor.
	virtual								~CP_DataEncoder();
	//@}
	
	//! @name Getters
	//@{
										/*!
										  Return the encoded or decoded data, depending on whether we called Encode or Decode().
										  \sa CP_HandleBlock.
										*/
			CP_HandleBlock				*GetEncodedData()								{ return( fData ); }

										/*!
										  Return the type of encoding being used, this should be a MIME type.
										  \sa CP_String.
										*/
			CP_String					GetEncodingType()								{ return( fEncodingType ); }

										/*!
										  Virtual method to return the size of the encoded data.
										  \sa CP_String.
										*/
	virtual	UInt32						GetEncodedSize( UInt32 inOriginalSize ) = 0;
	//@}

	//! @name Encode
	//@{
										/*!
										  Virtual method to actually do the encoding of the data, results are returned in the fData data member,
										  and the passed in data block is left untouched.
										  \sa CP_HandleBlock.
										*/
	virtual	bool						Encode( const CP_HandleBlock& inData ) = 0;

										/*!
										  Virtual method to actually do the decoding of the data, results are returned in the fData data member,
										  and the passed in data block is left untouched.
										  \sa CP_HandleBlock.
										*/
	virtual	bool						Decode( const CP_HandleBlock& inData ) = 0;

										/*!
										  Set the type of encoding being used, this should be a MIME type of description.
										  \sa CP_String.
										*/
			void						SetEncodingType( const CP_String& inEncoding )	{ fEncodingType = inEncoding; }
	//@}

	// Data members
protected:
	CP_String							fEncodingType;		/*! Describes the type of encoding being used, in MIME format */
	CP_HandleBlock						*fData;				/*! The encoded or decoded data. */
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DataEncoder
