//=========================================================================
//
//	FILE:				CP_Data_ResourceTypes.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions for the various data and resource types
//						used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Data_ResourceTypes
#define _H_CP_Data_ResourceTypes
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_OSTypes.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

//=========================================================================
// CPLAT default creator code
//=========================================================================
const FourCharCode kCP_CPLAT_CreatorCode				= CP_FOUR_CHAR_CODE('CPAP');

#if TARGET_OS_WIN32
//=========================================================================
// Folder types
//=========================================================================
	enum
	{
		kTemporaryFolderType         					 = CP_FOUR_CHAR_CODE('temp') /* temporary files go here (deleted periodically, but don't rely on it.) */
	};
#endif

#if TARGET_OS_LINUX
//=========================================================================
// Folder types
//=========================================================================
	enum
	{	
		kTemporaryFolderType         					 = CP_FOUR_CHAR_CODE('temp') /* temporary files go here (deleted periodically, but don't rely on it.) */
	};
#endif

//=========================================================================
// Data / Data File types
//=========================================================================
const UInt32 kCP_Unknown_FileType						= CP_FOUR_CHAR_CODE('\?\?\?\?');
const UInt32 kCP_Unknown_FileCreator					= CP_FOUR_CHAR_CODE('\?\?\?\?');

const UInt32 kCP_Text_FileType 							= CP_FOUR_CHAR_CODE('TEXT');

//=========================================================================
// Folder types
//=========================================================================
const FourCharCode kGenericFolderFolderType        		= CP_FOUR_CHAR_CODE('fold');

//=========================================================================
// Persistant Writer/Reader data types
//=========================================================================
const FourCharCode kCP_BinaryDataType					= CP_FOUR_CHAR_CODE('BINT');
const FourCharCode kCP_XMLDataType						= CP_FOUR_CHAR_CODE('XMLT');

//=========================================================================
// Resource types
//=========================================================================

#if TARGET_OS_MAC
	const FourCharCode kUnknownResourceType				= CP_FOUR_CHAR_CODE( '\?\?\?\?' );
	const FourCharCode kImageListResourceType			= CP_FOUR_CHAR_CODE( 'CPil' );
	const FourCharCode kMenuBarResourceType				= CP_FOUR_CHAR_CODE( 'MBAR' );
	const FourCharCode kMenuCommandTableResourceType	= CP_FOUR_CHAR_CODE( 'CPmc' );
	const FourCharCode kXMLDataResourceType				= CP_FOUR_CHAR_CODE( 'XML#' );
	const FourCharCode kPictureResourceType				= CP_FOUR_CHAR_CODE( 'PICT' );
	const FourCharCode kIconResourceType				= CP_FOUR_CHAR_CODE( 'icl8' );
#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32
	#if UNICODE
		#define kCLUT_ResourceType							L"CLUT"
		#define kImageListResourceType						L"CPil"

		#define kMenuBarResourceType						L"CPmb"
		#define kMenuCommandTableResourceType				L"CPmc"

			// Resource type defining string resources
		#define kStringListResourceType						L"STR#"
	#else
		#define kCLUT_ResourceType							"CLUT"
		#define kImageListResourceType						"CPil"

		#define kMenuBarResourceType						"CPmb"
		#define kMenuCommandTableResourceType				"CPmc"

			// Resource type defining string resources
		#define kStringListResourceType						"STR#"
	#endif
	
	const FourCharCode kXMLDataResourceType				= CP_FOUR_CHAR_CODE( 'XML#' );
#endif	// TARGET_OS_MAC

#if TARGET_OS_LINUX
	const FourCharCode kUnknownResourceType				= CP_FOUR_CHAR_CODE( '\?\?\?\?' );
	const FourCharCode kMenuResourceType				= CP_FOUR_CHAR_CODE( 'MENU' );
	const FourCharCode kMenuBarResourceType				= CP_FOUR_CHAR_CODE( 'MBAR' );
	const FourCharCode kMenuCommandTableResourceType	= CP_FOUR_CHAR_CODE( 'CPmc' );
	const FourCharCode kXMLDataResourceType				= CP_FOUR_CHAR_CODE( 'XML#' );
	const FourCharCode kPictureResourceType				= CP_FOUR_CHAR_CODE( 'XPMP' );
	const FourCharCode kIconResourceType				= CP_FOUR_CHAR_CODE( 'XPMI' );
#endif	// TARGET_OS_MAC

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Data_ResourceTypes

