//=========================================================================
//
//	FILE:				CP_DrawOSElements.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to draw the OS elements in CP_Views for custom
//						views that need to be drawn in the underlying OS style.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 243 $
//						Latest Checkout:  $Date: 2005-07-24 20:18:42 -0700 (Sun, 24 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_DrawOSElements
#define _H_CP_DrawOSElements
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Enums.h"
#include "CP_NoCopy.h"
#include "CP_Rect.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_DrawOSElements
*	@brief	A class to draw the OS elements in CP_Views for custom views that need to be drawn in the underlying OS style.
*/
class CP_EXPORT CP_DrawOSElements : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DrawOSElements(
												CP_View *inDestView,
												CP_Canvas& inDestCanvas );
	//! Destructor.
	virtual								~CP_DrawOSElements() throw();
	//@}
			
	//! @name Buttons
	//@{
			void						DrawButton(
												const CP_Rect& inBounds,
												SInt32 inValue,
												bool inIsActive,
												UInt32 inUserData );
			void						DrawListButton(
												const CP_Rect& inBounds,
												SInt32 inValue,
												bool inIsActive,
												UInt32 inUserData,
												bool inSortDirectionUp );
			void						DrawPushButton(
												const CP_Rect& inBounds,
												SInt32 inValue,
												bool inIsActive,
												UInt32 inUserData );
	//@}

	//! @name Frames
	//@{
			void						DrawFrame(
												const CP_Rect& inBounds,
												bool inPrimary,
												bool inIsActive );
	//@}

	//! @name Headers
	//@{
			void						DrawPlacard(
												const CP_Rect& inBounds,
												bool inIsActive );

			void						DrawWindowHeader(
												const CP_Rect& inBounds,
												bool inIsListHeader,
												bool inIsActive );
	//@}

	//! @name Hilite
	//@{
			void						DrawHilite(
												const CP_Rect& inBounds,
												bool inIsActive );
	//@}

	//! @name Misc
	//@{
			void						DrawDisclosure(
												const CP_Rect& inBounds,
												CP_E_Direction inDirection,
												bool inIsActive,
												UInt32 inUserData );
			void						DrawPopupArrow(
												const CP_Rect& inBounds,
												CP_E_Direction inDirection );
			void						DrawSeparator(
												const CP_Rect& inBounds,
												bool inIsActive );
			void						DrawSplitter(
												const CP_Rect& inBounds,
												bool inIsActive );
	//@}

	//! @name Text
	//@{

			void						DrawText(
												const CP_Rect& inBounds,
												const CP_String& inText,
												bool inIsActive,
												SInt32 inHorTextAlignment = CP_E_TextAlign_HorLeft,
												SInt32 inVerTextPlacement = CP_E_TextAlign_VerMiddle );
			CP_Size						MeasureText(
												const CP_String& inText,
												bool inIsActive );
	//@}

	//! @name Tracks
	//@{
			void						DrawSlider(
												const CP_Rect& inBounds,
												bool inIsActive,
												SInt32 inMin,
												SInt32 inMax,
												SInt32 inValue,
												UInt32 inUserData );
	//@}

protected:

	// class data
protected:
	CP_View								*fDestView;		/*! CP_View that is being drawn into. */
	CP_Canvas							fDestCanvas;	/*! Destination CP_Canvas that is used for the drawing. */
	
	#if TARGET_OS_MAC
		ThemeEraseUPP					fEraseUPP;		/*! TheEraseUPP for erasing the background. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Getters
	//@{
										/*!
										*	Gets the MacOS ThemeEraseUPP used to erase the background
										*	of drawn elements.
										*	
										*	@exception	none
										*	
										*	@return		ThemeEraseUPP	The ThemEraseUPP used to erase the
										*								background behind elements.
										*/
			ThemeEraseUPP				GetEraseUPP()									{ return( fEraseUPP ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the MacOS ThemeEraseUPP used to erase the background
										*	of drawn elements.
										*
										*	@param		ThemeEraseUPP	The ThemEraseUPP used to erase the
										*								background behind elements.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetEraseUPP( ThemeEraseUPP inEraseUPP )			{ fEraseUPP = inEraseUPP; }
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DrawOSElements

