//=========================================================================
//
//	FILE:				CP_MouseTracker.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle mouse tracking while the mouse button is down.
//	
//	HISTORY:			st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 191 $
//						Latest Checkout:  $Date: 2005-05-23 14:36:50 -0700 (Mon, 23 May 2005) $
//
//=========================================================================
#ifndef _H_CP_MouseTracker
#define _H_CP_MouseTracker
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Point.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

typedef void (*trackerProcPtr)(CP_View *inDestView, CP_Point& inMouseLocation, void *inData);

//!  CP_MouseTracker
/*!
  A class to handle tracking the mouse in a destination CP_View object. 
*/
class CP_EXPORT CP_MouseTracker
{
public:
								CP_MouseTracker(
										CP_View *inDestPane,
										trackerProcPtr inTrackingProcPtr,
										void *inData = NULL,
										trackerProcPtr inPreTrackingProcPtr = NULL,
										trackerProcPtr inPostTrackingProcPtr = NULL );
	virtual						~CP_MouseTracker();
	
									// Getters
								/*! Return the delay used before calling the OnTrackingMouse callback. */
			UInt32				GetTrackDelay()										{ return( fTrackingDelay ); }
								/*! Return any data associated with this object. */
			SInt32				*GetData()											{ return( (SInt32 *) fData ); }
								/*! Return the last location of the mouse. */
			CP_Point			GetLastMouseLocation()								{ return( fLastMouseLocation ); }
			
									// Setters
								/*! Set the tracking delay. */
			void				SetTrackDelay( UInt32 inTrackDelay )					{ fTrackingDelay = inTrackDelay; }
								/*! Set any data associated with this object. This data is passed to all of the tracking callbacks. */
			void				SetData( void *inData )								{ fData = inData; }
		
									// Mouse tracker
	virtual	void				TrackMouse( CP_Point& inStartLocation );

protected:
	virtual	void				OnPreTrackMouse( CP_Point& inLocation );
	virtual	void				OnTrackingMouse( CP_Point& inLocation );
	virtual	void				OnPostTrackMouse( CP_Point& inLocation );
	
	// class data
protected:
								/*! The destination pane we are tracking in. */
	CP_View						*fDestView;
								/*! Data associated with this object. */
	void						*fData;
								/*! The delay between tracking calls. */
	UInt32						fTrackingDelay;
	
								/*! Callback to a routine called just before we track the mouse. */
	trackerProcPtr				fPreTrackingProcPtr;
								/*! Callback to a routine called continuously during tracking. */
	trackerProcPtr				fTrackingProcPtr;
								/*! Callback to a routine called after tracking. */
	trackerProcPtr				fPostTrackingProcPtr;

								/*! Last location of the mouse. */
	CP_Point					fLastMouseLocation;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MouseTracker

