//=========================================================================
//
//	FILE:				CP_PersistantWriter.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for objects that are used with the CP_Persistant
//						class to write the CP_Persistant class data out.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_PersistantWriter
#define _H_CP_PersistantWriter
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Data_ResourceTypes.h"
#include "CP_FileSystemSpec.h"
#include "CP_NoCopy.h"
#include "CP_OSTypes.h"
#include "CP_Persistant.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_PersistantWriter
*	@brief	Base class for writing CP_Persistant class data.
*	
*	Base class for objects that are used with the CP_Persistant class to write
*	the CP_Persistant class data out
*/

class CP_EXPORT CP_PersistantWriter : public CP_NoCopy
{
public:
										// Construction / destruction
	virtual								~CP_PersistantWriter();
			
											// Getters
	virtual	FourCharCode				GetType()													{ return( kCP_Unknown_FileType ); }
	
											// Writing
	virtual CP_FileSystemSpec			Write(
												const CP_FileSystemSpec& inFileSpec,
												FourCharCode /*inType*/ )								{ return( inFileSpec ); }
									
protected:
											// Construction / destruction
										CP_PersistantWriter( CP_Persistant *inPersistant );

	// class data
protected:
	CP_Persistant 						*fPersistant;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PersistantWriter

