//=========================================================================
//
//	FILE:				CP_SmartPtr.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Template class that contains a pointer and correctly deletes it
///						when it goes out of scope.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_SmartPtr
#define _H_CP_SmartPtr
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_SmartPtr.h
*	@brief		Template class that contains a pointer and correctly deletes it when it goes out of scope
*/
template<class T> class CP_SmartPtr
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	explicit 							CP_SmartPtr( T *inPtr = NULL )						{ fPtr = inPtr; }
	//@}
	//! Destructor.
										~CP_SmartPtr();
	//@}

	//! @name operators
	//@{
			T&							operator *() const									{ return( *fPtr ); }
			T							*operator->() const									{ return( fPtr ); }

										operator const void *() const						{ return( fPtr ); }
	//@}

	//! @name Ptr management
	//@{
			T							*Get() const										{ return( fPtr ); }
			T							*Reset( T *inPtr);
			T							*Release();
	//@}

protected:

private:
										CP_SmartPtr( const CP_SmartPtr<T>& inPtr );
			CP_SmartPtr<T>& 			operator = ( const CP_SmartPtr<T>& inPtr );

	// class data
protected:
	T									*fPtr;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template<class T>
inline CP_SmartPtr<T>::~CP_SmartPtr()
{
	if( fPtr != NULL )
		delete fPtr;
}

template<class T>
T *CP_SmartPtr<T>::Reset( T *inPtr )
{
	if( fPtr != inPtr )
	{
		if( fPtr != NULL )
			delete fPtr;
		
		fPtr = inPtr;
	}
	
	return( fPtr );
}

template<class T>
T *CP_SmartPtr<T>::Release()
{
	T		*currentPtr = fPtr;
	
	fPtr = NULL;
	
	return( currentPtr );
}

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SmartPtr
