//=========================================================================
//
//	FILE:				CP_SplashScreen.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Splash screen class for the CPLAT framework.
//	
//	HISTORY:			1st Version - May 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev:$
//						Latest Checkout:  $Date:$
//
//=========================================================================
#ifndef _H_CP_SplashScreen
#define _H_CP_SplashScreen
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_DeferredTask.h"
#include "CP_SignalSlot.h"
#include "CP_TimedTask.h"
#include "CP_Window.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Application_Imp;

class CP_SplashScreen : public CP_M_DeferredTask, public CP_SigSlot::CP_SlotObject
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SplashScreen( CP_Application_Imp *inOwnerApp );
	virtual								~CP_SplashScreen() throw();
	//@}

	//! @name Instance
	//@{
	static	CP_SplashScreen				*Instance();
	//@}

	//! @name Getters
	//@{
			CP_Application_Imp			*GetOwnerApp()										{ return( fOwnerApp ); }
	//@}

	//! @name Setters
	//@{
			void						SetOwnerApp( CP_Application_Imp *inOwnerApp )		{ fOwnerApp = inOwnerApp; }
	//@}

	//! @name Window
	//@{
	virtual	void						Show( SInt32 inDuration );
	virtual	void						Show(
												SInt32 inWindowResID,
												SInt32 inDuration );
	virtual	void						Show(
												const CP_String& inNibName,
												const CP_String& inWindowName,
												SInt32 inDuration );
	virtual	void						Show(
												CP_Window *inSplashWindow,
												SInt32 inDuration );
	//@}

private:
	//! @name Slots
	//@{
			void						HandleTimer(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOSStatus );
	//@}

	// class data
private:
	static CP_SplashScreen				*fsInstance;		/*! Single instance of this class. */
	
protected:
	CP_Application_Imp					*fOwnerApp;			/*! Owner application class. */
	CP_Window							*fSplashWindow;		/*! Splash screen window. */

	CP_TimedTask						*fDurationTimer;	/*! Timer used to control length of time screen is shown. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SplashScreen

