//=========================================================================
//
//	FILE:				CP_XMLWriter_Element.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle elements used with our XML_Writer.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_XMLWriter_Element
#define _H_CP_XMLWriter_Element
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_HandleBlock.h"
#include "CP_FileSystemSpec.h"
#include "CP_NoCopy.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_Types.h"

	// MSL
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_EXPORT CP_XMLWriter;
class CP_EXPORT CP_XMLWriter_Entry;
class CP_EXPORT CP_XMLWriter_ElementAttribute;
class CP_EXPORT CP_XMLWriter_Element;

/*!
*	@class		CP_XMLWriter_ElementAttribute
*	@brief		This class sets the attributes for a specific CP_XMLWriter_Element to be
*				written out as XML data.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLWriter_ElementAttribute
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLWriter_ElementAttribute();
										CP_XMLWriter_ElementAttribute( const CP_String& inAttributeTag );
										CP_XMLWriter_ElementAttribute( const CP_XMLWriter_ElementAttribute& inAttribute );
	//! Destructor.
	virtual								~CP_XMLWriter_ElementAttribute();
	//@}

	//! @name Attributes
	//@{
			CP_String					GetAttributeTag()								{ return( fAttributeTag ); }
			CP_String					GetAttributeData()								{ return( fAttributeData ); }
	//@}

	//! @name Operators
	//@{
			CP_XMLWriter_ElementAttribute&	operator =( const CP_XMLWriter_ElementAttribute& inAttribute );
	//@}

	//! @name Setters
	//@{
			void						SetValue( bool inValue );
			void						SetValue( SInt16 inValue );
			void						SetValue( UInt16 inValue );
			void						SetValue( SInt32 inValue );
			void						SetValue( UInt32 inValue );
			void						SetValue( const CP_String& inValue );
			void						SetValue( char inValue );
			void						SetValue( float inValue );
			void						SetValue( const CP_RGBColor& inValue );
			void						SetValue( const CP_Point& inValue );
			void						SetValue( const CP_Size& inValue );
			void						SetValue( const CP_Rect& inValue );
			void						SetValue( const CP_FileSystemSpec& inValue );
			
			void						SetAttributeTag( const CP_String& inTag )		{ fAttributeTag = inTag; }
			void						SetAttributeData( const CP_String& inData )		{ fAttributeData = inData; }
	//@}

protected:
	//! @name Formatting
	//@{
	virtual	CP_String					FormatStringForXML( const CP_String& inOutString );
	//@}

	// class data members
protected:
	CP_String							fAttributeTag;		/*! The XML tag for the data. This is the tag that you
    														use to retrieve the data with. */
    								
	CP_String							fAttributeData;		/*! The data of the attribute. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

	// typedefs
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_XMLWriter_Entry *, std::allocator<CP_XMLWriter_Entry *> >										CP_XMLElements_Array;
	typedef std::vector<CP_XMLWriter_Entry *, std::allocator<CP_XMLWriter_Entry *> >::iterator								CP_XMLElements_Iterator;
	typedef std::vector<CP_XMLWriter_Entry *, std::allocator<CP_XMLWriter_Entry *> >::reverse_iterator						CP_XMLElements_ReverseIterator;

	typedef std::vector<CP_XMLWriter_ElementAttribute, std::allocator<CP_XMLWriter_ElementAttribute> >						CP_XMLAttributes_Array;
	typedef std::vector<CP_XMLWriter_ElementAttribute, std::allocator<CP_XMLWriter_ElementAttribute> >::iterator			CP_XMLAttributes_Iterator;
	typedef std::vector<CP_XMLWriter_ElementAttribute, std::allocator<CP_XMLWriter_ElementAttribute> >::reverse_iterator	CP_XMLAttributes_ReverseIterator;
#else
	typedef std::vector<CP_XMLWriter_Entry *>																				CP_XMLElements_Array;
	typedef std::vector<CP_XMLWriter_Entry *>::iterator																		CP_XMLElements_Iterator;
	typedef std::vector<CP_XMLWriter_Entry *>::reverse_iterator																CP_XMLElements_ReverseIterator;

	typedef std::vector<CP_XMLWriter_ElementAttribute>																		CP_XMLAttributes_Array;
	typedef std::vector<CP_XMLWriter_ElementAttribute>::iterator															CP_XMLAttributes_Iterator;
	typedef std::vector<CP_XMLWriter_ElementAttribute>::reverse_iterator													CP_XMLAttributes_ReverseIterator;
#endif

/*!
*	@class		CP_M_XMLEntry_Attributes
*	@brief		Mix-in class for XMLWriter element entries tha need to handle writing a list
*				of attributes.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_M_XMLEntry_Attributes : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_XMLEntry_Attributes();
	//! Destructor.
	virtual								~CP_M_XMLEntry_Attributes();
	//@}
	
	//! @name Attributes
	//@{
			void						AddAttribute( const CP_XMLWriter_ElementAttribute& inAttribute );
			UInt32						GetNumberAttributes();
			CP_XMLWriter_ElementAttribute	GetNthAttribute( UInt32 inIndex );
	//@}

protected:
	//! @name Attributes
	//@{
			void						AddAttributesToHeader( CP_String& inHeader );
	//@}

	// class data members
protected:
	CP_XMLAttributes_Array				*fAttributes;		/*! The list of attributes being written. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_M_XMLEntry_Attributes
*	@brief		Base class for entries used with the CP_XMLWriter class to write XML data to a stream.
*
*				To add an XML entry you need to create an instance of one of:
*					- CP_XMLWriter_Element
*					- CP_XMLWriter_Comment
*					- CP_XMLWriter_ProcessingInstruction
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLWriter_Entry : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_XMLWriter_Entry();
	//@}
	
	//! @name XMLWriter
	//@{
			CP_XMLWriter				*GetXMLWriter()										{ return( fXMLWriter ); }
	//@}

	//! @name Data and Tags
	//@{
			virtual void				SetData( const CP_String& inData )					{ fData = inData; }
			virtual const CP_String 	GetData() const									{ return fData; }
			void						SetTag( const CP_String& inTag )					{ fTag = inTag; }
	//@}
			
	//! @name Child Elements
	//@{
			void						AddChildElement( CP_XMLWriter_Entry *inChildElement );
			UInt32						GetElementsDepth();
			UInt32						GetNumberChildElements();
			CP_XMLWriter_Entry			*GetNthChildElement( UInt32 inIndex );
	//@}

	//! @name Testers
	//@{
			bool						IsWritten()											{ return( fIsWritten ); }
	//@}
			
	//! @name Writing
	//@{
			void						Write( CP_XMLWriter *inXMLWriter = NULL );

	virtual	void						WriteHeader( CP_XMLWriter */*inXMLWriter = NULL*/ )	{}
	virtual	void						WriteTrailer( CP_XMLWriter */*inXMLWriter = NULL*/ ){}
	virtual	void						WriteData( CP_XMLWriter */*inXMLWriter = NULL*/ )	{}
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Contructor.
										CP_XMLWriter_Entry(
												CP_XMLWriter *inXMLWriter,
												CP_XMLWriter_Entry *inParentElement,
												const CP_String& inTag,
												const CP_String& inData );
	//@}

	// class data members
protected:
 	CP_XMLWriter						*fXMLWriter;		/*! The XMLWriter that we are going to write the data to. */
	CP_XMLWriter_Entry					*fParentElement;	/*! The parent element that this entry goes with. */
	CP_String							fTag;				/*! The tag for this entry. */
 	CP_String							fData;				/*! The actual data for the entry. */
	CP_XMLElements_Array				*fElements;			/*! A list of child elements for this entry. */
	bool								fIsWritten;			/*! Flag to indicate if the data has been written. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

	// type of element data we have in our element
enum
{
	CP_E_ElementData_NoData,		/*!< Element contains no data. */
	CP_E_ElementData_CharData,		/*!< Element contains character data. */
	CP_E_ElementData_CData			/*!< Element contains CData data. */
};

/*!
*	@class		CP_XMLWriter_Element
*	@brief		XML entry class that represents an element in an XML file. This is the main class
*				for writing data to an XML file. An element can have children and attributes.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLWriter_Element : public CP_XMLWriter_Entry, public CP_M_XMLEntry_Attributes
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLWriter_Element(
												CP_XMLWriter *inXMLWriter,
												CP_XMLWriter_Element *inParentElement,
												const CP_String& inTag,
												const CP_String& inData = "" );
	//! Destructor.
	virtual								~CP_XMLWriter_Element();
	//@}

	//! @name CData
	//@{
			void						AddCData( CP_HandleBlock *inHandleBlock );
			void						AddCData( const CP_String& inStringData );
	//@}

	//! @name Getters
	//@{
			SInt16						GetElementDataType()								{ return( fElementDataType ); }
	//@}

	//! @name Setters
	//@{
	virtual void						SetData( const CP_String& inData );
	//@}

	//! @name Writers
	//@{
	virtual	void						WriteHeader( CP_XMLWriter *inXMLWriter = NULL );
	virtual	void						WriteTrailer( CP_XMLWriter *inXMLWriter = NULL );
	virtual	void						WriteData( CP_XMLWriter *inXMLWriter = NULL );
	//@}

protected:
	//! @name Formatters
	//@{
	virtual	CP_String					FormatStringForXML( const CP_String& inOutString );
	//@}

private:
	//! @name CData
	//@{
			void						WriteCData(CP_XMLWriter* inXMLWriter);
	//@}
			

	// class data members
protected:
	CP_HandleBlock						*fCData;			/*! If the element contains CData, then this object contains
    													the data to be written. */

	SInt16								fElementDataType;	/*! The type of data for this element. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_XMLWriter_ProcessingInstruction
*	@brief		XML entry class that represents a processing instruction in an XML file.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLWriter_ProcessingInstruction : public CP_XMLWriter_Entry, public CP_M_XMLEntry_Attributes
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLWriter_ProcessingInstruction(
												CP_XMLWriter *inXMLWriter,
												CP_XMLWriter_Element *inParentElement,
												const CP_String& inTag,
												const CP_String& inData );
	//! Destructor.
	virtual								~CP_XMLWriter_ProcessingInstruction();
	//@}

	//! @name Writers
	//@{
	virtual	void						WriteHeader( CP_XMLWriter *inXMLWriter = NULL );
	virtual	void						WriteTrailer( CP_XMLWriter */*inXMLWriter = NULL*/ )								{}
	virtual	void						WriteData( CP_XMLWriter */*inXMLWriter = NULL*/ )									{}
	//@}

	// class data members
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_XMLWriter_Element


