//=========================================================================
//
//	FILE:				CP_AE_Record.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Wrapper class for dealing with Apple Event records.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_AE_Record
#define _H_CP_AE_Record
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_AE_List.h"
#include "CP_FileSystemSpec.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_AE_Record
*	@brief		The CP_AE_Record class is a MacOS only class to deal with Apple Event
*				records, and is a subclass of CP_AE_List.
*
*	@ingroup	AppleEvents
*/
class CP_EXPORT CP_AE_Record : public CP_AE_List
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AE_Record();
										CP_AE_Record( const AEDescList& inDesc );
	//! Destructor.
	virtual								~CP_AE_Record();
	//@}

	//! @name Getters
	//@{
			OSErr						GetNthPtr(
												UInt32 inIndex,
												DescType inDesiredType,
												void *inData,
												SInt32 inBytes,
												DescType *inKeyword,
												DescType *inTypeCode,
												SInt32 *inOutActualBytes ) const;
			OSErr						GetKey(
												AEKeyword inKeyword,
												DescType desiredType,
												AEDesc *inOutDescriptor ) const;
			OSErr						GetKey(
												AEKeyword inKeyword,
												DescType inDesiredType,
												void *inData,
												SInt32 inBytes) const;
			OSErr						GetKey(
												AEKeyword inKeyword,
												DescType inDesiredType,
												DescType *inTypeCode,
												void *inData,
												SInt32 inBytes,
												SInt32 *inOutActualBytes ) const;

			bool						GetBoolean( AEKeyword inKey ) const;
			SInt16						GetSInt16( AEKeyword inKey ) const;
			SInt32						GetSInt32( AEKeyword inKey ) const;
			float						GetFloat( AEKeyword inKey ) const;
			double						GetDouble( AEKeyword inKey ) const;
			CP_Rect						GetRect( AEKeyword inKey ) const;
			CP_RGBColor					GetColor( AEKeyword inKey ) const;
			ProcessSerialNumber			GetPSN( AEKeyword inKey ) const;
			CP_FileSystemSpec					GetFileSpec( AEKeyword inKey ) const;
			DescType					GetEnum( AEKeyword inKey ) const;
			AEDesc			GetDesc(
												AEKeyword inKey,
												DescType inDesiredType ) const;
	//@}

	//! @name Setters
	//@{
			void						Append(
												AEKeyword inKeyword,
												DescType inTypeCode,
												const void *inData,
												UInt32 InBytes );
			void						Append(
												AEKeyword inKeyword,
												const AEDesc& inDesc );

			void						SetKey(
												AEKeyword inKeyword,
												DescType inTypeCode,
												const void *inData,
												SInt32 inBytes );
			void						SetKey(
												AEKeyword keyword,
												const AEDesc &inDescriptor );
			void						SetBoolean(
												AEKeyword inKey,
												bool inValue );
			void						SetSInt16(
												AEKeyword inKey,
												SInt16 inValue );
			void						SetSInt32(
												AEKeyword inKey,
												SInt32 inValue );
			void						SetFloat(
												AEKeyword inKey,
												float inValue );
			void						SetDouble(
												AEKeyword inKey,
												double inValue );
			void						SetRect(
												AEKeyword inKey,
												const CP_Rect& inRect );
			void						SetColor(
												AEKeyword inKey,
												const CP_RGBColor& inColor );
			void						SetPSN(
												AEKeyword inKey,
												const ProcessSerialNumber& inValue );
			void						SetFilespec(
												AEKeyword inKey,
												const CP_FileSystemSpec& inValue );
			void						SetDesc(
												AEKeyword inKey,
												const AEDesc& inValue );
			void						SetEnum(
												AEKeyword inKey,
												const AEDesc& inValue );

			void						DeleteKey( AEKeyword inKeyword );

											// Testers
			bool						HasKeyword(
												AEKeyword inKeyword,
												DescType inDesiredType ) const;
	//@}

protected:

	// class data members
protected:
};

CPLAT_End_Namespace_CPLAT

#endif

#endif	// _H_CP_AE_Record

