//=========================================================================
//
//	FILE:				CP_Graphics_Imp_Quartz.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Quartz implementation of the CP_Graphics object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 173 $
//						Latest Checkout:  $Date: 2005-05-15 12:38:04 -0700 (Sun, 15 May 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp_Quartz
#define _H_CP_Graphics_Imp_Quartz
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_Graphics_Imp.h"
#include "CP_Graphics_Imp_Quartz_defs.h"

#if TARGET_API_MAC_CARBON && (UNIVERSAL_INTERFACES_VERSION >= 0x0335)
	#if defined(__CF_USE_FRAMEWORK_INCLUDES__)
		#include <Carbon/Carbon.h>				// Carbon framework
		#include <QuickTime/ImageCompression.h>	// Quicktime framework
	#else
		#include <Carbon.h>						// Universal Headers 3.4 and above / non Carbon framework
		#include <ImageCompression.h>
	#endif
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Graphics_Imp_Quartz
*	@brief		CP_Graphics_Imp subclass to perform drawing using Quartz.
*
*				This is the CP_Graphics_Imp class used by a CP_Graphics object when the
*				user has selected Quartz as the underlying graphics engine. This will
*				also be selected as the native platform type when running under MacOSX
*				Mach with the Quartz drawing available.
*/
class CP_EXPORT CP_Graphics_Imp_Quartz : public CP_Graphics_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Graphics_Imp_Quartz(
												CP_Canvas& inDestCanvas,
												bool inSynchronizeWithPort = true );
	//! Destructor.
	virtual								~CP_Graphics_Imp_Quartz();
	//@}

	//! @name Context
	//@{
			CGContextRef				GetCGContext()														{ return( fContextRef ); }
	//@}

	//! @name Setters
	//@{
	virtual	void						SetAlpha( float inAlpha );
	virtual	void						SetLineWidth( float lineWidth );
	virtual	void						SetLineJoin( CP_E_GraphicsLineJoin inLineJoin );
	virtual	void						SetLineCap( CP_E_GraphicsLineCap inLineCap );
	virtual	void						SetStrokeColor( CP_ARGBColor& inStrokeColor );
	virtual void						SetFillColor( CP_ARGBColor& inFillColor );
	//@}
			
	//! @name Graphics State
	//@{
	virtual	void						SaveState();
	virtual	void						RestoreState();
	
	virtual	void						ScaleCTM(
											float inHorAmt,
											float inVerAmt );
	virtual	void						TranslateCTM(
											float inHorAmt,
											float inVerAmt );
	virtual	void						RotateCTM( float inAngle );
	//@}

	//! @name Arcs
	//@{
	virtual	void						StrokeArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
	virtual void						FillArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
	//@}

	//! @name Bezier Curves
	//@{
	virtual	void						StrokeBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inControlPointTwo,
											const CP_Point& inEndPoint );
	virtual	void						StrokeQuadBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inEndPoint );
	//@}

	//! @name Gradient
	//@{
	virtual	void						DrawGradient(
												CP_E_GradientType inGradientType,
												CP_Rect& inDestBounds );
	virtual	void						SetGradientAngle( SInt16 inAngle )						{ fGradientAngle = inAngle; }
	virtual	void						SetGradientColor(
												const SInt16 inIndex,
												const CP_ARGBColor& inColour );
	virtual	void						SetGradientCount( SInt16 inGradientCount )				{ fGradientCount = inGradientCount; }
	//@}

	//! @name Icons
	//@{
	virtual	void						DrawIcon(
												const CP_Rect& inBounds,
												CP_E_IconStyle inIconType,
												SInt16 inIconID );
	virtual	void						DrawIcon(
												CP_OSIcon& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Images
	//@{
	virtual	void						DrawImage(
												SInt16 inImageResID,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inSourceBounds,
												CP_Rect& inDestBounds );
	//@}

	//! @name Lines
	//@{
	virtual	void						Line(
												const CP_Point& inPoint,
												const CP_Size& inSize );
	virtual	void						Line(
												float inStartHorLocation,
												float inStartVerLocation,
												float inEndHorLocation,
												float inEndVerLocation );
	//@}

	//! @name Paths
	//@{
	virtual	void						BeginPath();
	virtual	void						ClosePath();
	virtual	void						StrokePath();
	virtual	void						FillPath();
	virtual	void						DrawPath( CP_E_PathDrawingMode inPathDrawMode );
	virtual	void						PathMoveTo( const CP_Point& inPoint );
	virtual	void						PathMoveTo(
												float inHorLocation,
												float inVerLocation );
	virtual	void						PathAddLineTo( const CP_Point& inPoint );
	virtual	void						PathAddLines(
												const CP_Point inPoint[],
												SInt16 inNumberPoints );
	virtual	void						PathAddLine(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint );
	virtual	void						PathAddLine(
												float inStartH,
												float inStartV,
												float inEndH,
												float inEndV );
	virtual	void						PathAddRect( const CP_Rect& inBounds );
	virtual	void						PathAddBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inControlPointTwo,
											const CP_Point& inEndPoint );
	virtual	void						PathAddQuadBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inEndPoint );
	virtual	void						PathAddArc(
												float x,
												float y,
												float radius,
												float startAngle,
												float endAngle,
												SInt16 clockwise );
	virtual	void						PathAddArcToPoint(
												float x1,
												float y1,
												float x2,
												float y2,
												float radius );
	virtual	void						PathAddOval( const CP_Rect& inBounds );
	virtual	void						PathAddPoly( CP_Point_Array& inPoints );
	virtual	void						ClipPath();
	//@}
									
	//! @name Polygons
	//@{
	virtual	void						StrokePoly( CP_Point_Array& inPoints );
	virtual	void						FillPoly( CP_Point_Array& inPoints );
	//@}

	//! @name Rects
	//@{
	virtual	void						StrokeRect( const CP_Rect& inBounds );
	virtual	void						FillRect( const CP_Rect& inBounds );
	//@}

	//! @name Regions
	//@{
	virtual	void						StrokeRegion( const CP_Region& inRegion );
	virtual	void						FillRegion( const CP_Region& inRegion );
	//@}

	//! @name Round Rects
	//@{
	virtual	void						StrokeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	virtual	void						FillRoundRect( 
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	//@}

	//! @name Oval
	//@{
	virtual	void						StrokeOval( const CP_Rect& inBounds );
	virtual	void						FillOval( const CP_Rect& inBounds );
	//@}

	//! @name Text
	//@{
	virtual	void						DrawString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
	virtual	void						DrawQuartzString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
	virtual	void						DrawText(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop,
												bool inWrapText = true );
	virtual	void						DrawTrunc(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop );
	//@}

protected:

private:
	//! @name Misc
	//@{
			void						Init();

			void						MakeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
			void						MakeOval(
												CGPoint inCenter,
												float a,
												float b );
	//@}

	//! @name Gradient
	//@{
			void						DrawGradient_imp(
												CP_E_GradientType inGradietType,
												CP_Rect& inDestBounds );

	virtual	void						BuildGradient(
												CP_Rect& inDestBounds,
												CP_Point& inCenterPoint );

			void						InitGradient();

			CP_E_GradientType			GetGradiientType()										{ return( fGradientType ); }
			SInt16						GetGradientAngle()										{ return( fGradientAngle ); }
			SInt16						GetNumberGradients()									{ return( fGradientCount ); }

			SInt32						CalcGradientRange(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						CalcGradientColors(
												SInt32 inRange,
												CP_RGBColor& inOutColorOne,
												CP_RGBColor& inOutColorTwo,
												CP_E_GradientColorInfo *inOutGradientInfo );	
			void						AdjustColor(
												CP_RGBColor& inOutColor,
												CP_E_GradientColorInfo *inOutGradientInfo );
			void						CalcEnclosingBounds(
												CP_Rect& inOutBounds,
												CP_Point& inCenterPoint );	

			void						BuildRect(
												CP_Rect& inDestBounds,
												CP_Point& inCenterPoint,
												CP_E_GradientType inGradientType );
			void						BuildRadial(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						BuildSunburst(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint,
												CP_E_GradientType inGradientType );
			void						BuildShapeburst(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						BuildAngled( CP_Rect& inBounds );

			UInt16						CP_FixedToColor( CP_Fixed inValue );
	//@}
									
	//! @name Images
	//@{
			void						ReadBitmapInfo(
												GraphicsImportComponent gi,
												CP_S_QuartzBitmapInfo *bi );
			void						GetBitmapData(
												GraphicsImportComponent gi,
												CP_S_QuartzBitmapInfo *bi );
	//@}

	//! @name Polygons
	//@{
			void						CreatePolyShape();
	//@}

	// class data
protected:
	CGContextRef						fContextRef;
	CFBundleRef							fAppServicesFrameworkBundle;
	
		// cached function pointers
	CGContextSetAlphaFuncPtr			fSetAlphaFP;
	CGContextSetLineWidthFuncPtr		fSetLineWidthFP;
	CGContextSetLineJoinFuncPtr			fLineJoinFP;
	CGContextSetLineCapFuncPtr			fLineCapFP;
	CGContextSetRGBStrokeColorFuncPtr	fStrokeColorFP;
	CGContextSetRGBFillColorFuncPtr		fFillColorFP;
	
		// Graphics State
	CGContextSaveGStateFuncPtr			fSaveGStateFP;
	CGContextRestoreGStateFuncPtr		fRestoreGStateFP;
	
	CGContextScaleCTMFuncPtr			fScaleCTMFP;
	CGContextTranslateCTMFuncPtr		fTranslateCTMFP;
	CGContextRotateCTMFuncPtr			fRotateCTMFP;
	
		// Bezier
	CGContextAddCurveToPointFuncPtr		fCurveToFP;
	CGContextAddQuadCurveToPointFuncPtr	fQuadCurveToFP;
			
		// ColorSpace
	CGColorSpaceCreateDeviceRGBFuncPtr	fColorSpaceCreateDeviceRGBFP;
	CGColorSpaceReleaseFuncPtr			fColorSpaceReleaseFP;
	
		// Lines
	CGContextMoveToPointFuncPtr			fMoveToFP;
	CGContextAddLineToPointFuncPtr		fLineToFP;
	
		// Image
	CGDataProviderReleaseFuncPtr		fDataProviderReleaseFP;
	CGDataProviderCreateWithDataFuncPtr	fDataProviderCreateWithDataFP;
	CGImageReleaseFuncPtr				fImageReleaseFP;
	CGContextDrawImageFuncPtr			fContextDrawImageFP;
	CGImageCreateFuncPtr				fImageCreateFP;
			
		// Paths
	CGContextBeginPathFuncPtr			fBeginPathFP;
	CGContextClosePathFuncPtr			fClosePathFP;
	CGContextStrokePathFuncPtr			fStrokePathFP;
	CGContextFillPathFuncPtr			fFillPathFP;
	CGContextDrawPathFuncPtr			fDrawPathFP;
	CGContextAddRectFuncPtr				fAddRectFP;
	CGContextAddArcFuncPtr				fAddArcFP;
	CGContextAddArcToPointFuncPtr		fAddArcToPointFP;
	CGContextClipFuncPtr				fClipFP;
		
		// Polygons
	CP_Point_Array						fPoints;

		// Rects
	CGContextFillRectFuncPtr			fFillRectFP;
	CGContextStrokeRectFuncPtr			fStrokeRectFP;
	
		// Text
	CGContextShowTextAtPointFuncPtr		fShowTextAtPoint;
	CGContextSelectFontFuncPtr			fSelectFont;
	CGAffineTransformScaleFuncPtr		fAffineTransformScale;
	CGContextSetTextMatrixFuncPtr		fSetTextMatrix;

		// Gradient
	CP_E_GradientType					fGradientType;
	SInt16								fGradientCount;
	CP_RGBColor							fColors[ kCP_MaxGradientColors ];
	SInt16								fOffsets[ kCP_MaxGradientColors ];
	SInt16								fGradientAngle;
	CP_Region							fDestRegion;
};

CPLAT_End_Namespace_CPLAT

#endif	// defined(TARGET_API_MAC_CARBON)

#endif	// _H_CP_Graphics_Imp_Quartz
