//=========================================================================
//
//	FILE:				CP_HIView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_HIObject subclass for our base class of custom
//						HIView objects.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_HIView
#define _H_CP_HIView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2

	// CPLAT_Framework
#include "CP_HIObject.h"
#include "CP_Point.h"
#include "CP_Size.h"

#ifdef __CF_USE_FRAMEWORK_INCLUDES__
	#include <Carbon/Carbon.h>
#endif	// __CF_USE_FRAMEWORK_INCLUDES__

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_HIView
*	@brief		Class to provide HIView support to CPLAT.
*
*	@ingroup	UI
*/
class CP_HIView : public CP_HIObject
{
public:
	//! @name Getters
	//@{
								/*!
								*	Returns the HIViewRef for this object
								*	
								*	@exception	none
								*	
								*	@return		HIViewRef		MacOS HIViewRef for this object.
								*/
			HIViewRef			GetHIViewRef() const								{ return( (HIViewRef) GetObjectRef() ); }
			HIRect				Bounds();
	//@}
	
	//! @name Appearance
	//@{
			void				Show();
			void				Hide();

			void				Activate();
			void				Deactivate();

			void				Enable();
			void				Disable();
	//@}

	//! @name Events
	//@{
	virtual	void				InitEventHandler( CP_EventHandler& /*inEventHandler*/ )	{}
	//@}

	//! @name Setters
	//@{
			void				SetBounds(
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize );
	virtual	void				SetLocation( const CP_Point& inLocation );
	virtual	void				SetSize( const CP_Size& inSize );
	//@}

	//! @name Subviews
	//@{
			void				AddSubview( CP_HIView *inSubview );
	//@}

	//! @name Testers
	//@{
			bool				IsVisible() const;
			bool				IsActive() const;
			bool				IsEnabled() const;
	//@}

protected:
		// Autoinvalidation
		enum					{ 
									kAutoInvalidateOnActivate 		= (1 << 0),
									kAutoInvalidateOnHilite			= (1 << 1),
									kAutoInvalidateOnEnable			= (1 << 2),
									kAutoInvalidateOnValueChange	= (1 << 3),
									kAutoInvalidateOnTitleChange	= (1 << 4)
								};
								
	//! @name Invalidation
	//@{
			void				ChangeAutoInvalidateFlags(
									OptionBits inSetFlags,
									OptionBits inClearFlags );

								/*!
								*	Returns the auto invalidation flags for this object. These flags
								*	determine which action will cause automatic invalidation of this
								*	object.
								*	
								*	@exception	none
								*	
								*	@return		OptionBits		Invalidation flags to return.
								*/
			OptionBits			GetAutoInvalidateFlags()								{ return( fAutoInvalidateFlags ); }
	//@}
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_HIView( HIViewRef inView );
	//! Destructor.
	virtual						~CP_HIView() throw();
	//@}

#if 0
	//! @name Drag and Drop
	//@{
		virtual bool			DragEnter( DragRef inDrag );
		virtual bool			DragWithin( DragRef inDrag );
		virtual bool			DragLeave( DragRef inDrag );
		virtual OSStatus		DragReceive( DragRef inDrag );
	//@}
#endif

	//! @name Drawing
	//@{
	virtual void				Draw(
									RgnHandle inLimitRgn,
									CGContextRef inContext );
	//@}

	//! @name Getters
	//@{
								/*!
								*	Returns the ControlKind definition for this object. Subclasses
								*	must override this pure virtual function in their classes.
								*	
								*	@exception	none
								*	
								*	@return		ControlKind		Description of the kind of control this is.
								*/
	virtual ControlKind			GetKind() = 0;
	virtual UInt32				GetBehaviors();
	virtual ::OSStatus			GetRegion(
										ControlPartCode inPart,
										RgnHandle outRgn );
	//@}
	
	//! @name Registration
	//@{
								/*!
								*	Registers this class with the HIObject system for creation.
								*
								*	@param		inID			Class id
								*	@param		inProc			ConstructProc used to construct this class.
								*	
								*	@exception	none
								*	
								*	@return		::OSStatus		Result of the registration.
								*/
	static ::OSStatus			RegisterSubclass(
										CFStringRef inID,
										ConstructProc inProc )							{ return( CP_HIObject::RegisterSubclass( inID, kHIViewClassID, inProc ) ); }
	//@}
		
	//! @name Debugging
	//@{
	virtual void				PrintDebugInfoSelf();
	//@}
	
	// class data
private:
	OptionBits					fAutoInvalidateFlags;	/*! Flags to indication when this view is automatically invalidated. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2

#endif // _H_CP_HIView
