//=========================================================================
//
//	FILE:				CP_Window_Imp_CE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS CarbonEvents implemention of the CP_Window class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 275 $
//						Latest Checkout:  $Date: 2005-09-10 21:42:34 -0700 (Sat, 10 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_Window_Imp_CE
#define _H_CP_Window_Imp_CE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_EventHandler.h"
#include "CP_M_CommandHandler.h"
#include "CP_OSIcon.h"
#include "CP_OSStatus.h"
#include "CP_Point.h"
#include "CP_ScriptableObject.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_View.h"
#include "CP_WindowTypes.h"

CPLAT_Begin_Namespace_CPLAT

enum
{
	pIsMinimizable	= 'ismn',	// Can the window be minimized ?
	pIsMinimized	= 'pmnd'	// Is the window currently minimized ?
};

/*!
*	@class		CP_Window_Imp
*	@brief		Implementation of the CP_Window class for Carbon Events.
*
*				The implementation provides the CP_Window class under MacOS
*				using Carbon and Carbon Event handling.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Window_Imp : public CP_View, public CP_ScriptableObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Window_Imp();
										CP_Window_Imp( WindowRef inOSWindow );
	//! Destructor.
	virtual								~CP_Window_Imp() throw();
	//@}
				
	//! @name Appearance
	//@{
	virtual	void						Show();
	virtual	void						Hide();
	
			void						Select();
	//@}
			
	//! @name Apple Events
	//@{
	virtual	void						DoAE_GetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );
	virtual	void						DoAE_SetData(
												const DescType inPropertyType,
												const AEDesc& inValue,
												AppleEvent& inOutReply );

			bool						IsCollapseable() const;
			bool						IsCollapsed() const;
			bool						IsCloseable() const;
			bool						IsFloating() const;
			bool						IsModal() const;
			bool						IsResizeable() const;
			bool						IsZoomable() const;
			bool						IsZoomed() const;
	//@}

	//! @name Commands
	//@{
			void						HandleCloseUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );

	virtual	void						HandleCloseCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Conversions
	//@{
	virtual	void						PortToGlobal( CP_Point& inPoint );
	virtual	void						PortToGlobal( CP_Rect& inRect );
	virtual	void 						GlobalToPort( CP_Point& inPoint );
	virtual	void						GlobalToPort( CP_Rect& inRect );
	//@}

			void						Flush();

	//! @name Data
	//@{
	virtual	OSStatus					GetData(
												UInt32 inTagName,
												SInt32 inDataSize,
												void *inData );
	virtual	void						SetData(
												UInt32 inTagName,
												SInt32 inDataSize,
												const void *inData );
	//@}

	//! @name Drawing
	//@{
	virtual	void						Refresh();
	//@}

	//! @name Events
	//@{
			void						SendCloseEvent();
			
			void						InstallMouseMovedEventHandler();
	//@}

	//! @name Getters
	//@{
	virtual	CP_Rect						GetLocalBounds();
	virtual	CP_Rect						GetPortBounds();
	virtual	CP_Rect						GetContentBounds();
	virtual	CP_Rect						GetGlobalBounds();
	virtual	CP_Size						GetSize() const;
	virtual	CP_Point					GetLocation() const;

	#if ! TARGET_RT_MAC_MACHO
										/*!
										*	Return the update region for this window. This is the region we save off in the
										*	update event before drawing the window contents.
										*	
										*	@exception	none
										*
										*	@return		CP_Region			The current update region for the window.
										*/
		virtual	CP_Region				GetUpdateRegion()										{ return( fUpdateRegion ); }
	#endif	// ! TARGET_RT_MAC_MACHO

										/*!
										*	Return the minimum size a window can shrink to.
										*	
										*	@exception	none
										*
										*	@return		CP_Size const		The minimum size a window is allowed to shrink to.
										*/
			CP_Size						GetMinSize() const										{ return( fMinSize ); }

										/*!
										*	Return the maximum size a window can grow to.
										*	
										*	@exception	none
										*
										*	@return		CP_Size const		The maximumm size a window is allowed to grow to.
										*/
			CP_Size						GetMaxSize() const										{ return( fMaxSize ); }

										/*!
										*	Return the ideal size we want this window to be.
										*	
										*	@exception	none
										*
										*	@return		CP_Size const		The ideal size of the window.
										*/
			CP_Size						GetIdealSize() const									{ return( fIdealSize ); }

										/*!
										*	Returns the MacOS WindowRef that this window object is using for its platform window.
										*
										*	@exception	none
										*
										*	@return		OSWindow			MacOS WindowRef that implements the window.
										*/
			OSWindow					GetImpPlatformWindow() const							{ return( fOSWindow ); }
	//@}

	//! @name Zoom
	//@{
			void						Zoom( bool inZoomOutFlag );
	//@}

	//! @name Proxy Icon
	//@{
			void						SetProxyIcon( CP_OSIcon *inOSIcon );
			void						SetProxyCreatorType(
												FourCharCode inCreator,
												FourCharCode inType );
			void						SetProxyFileSpec( const CP_FileSystemSpec& inFileSpec );
			void						RemoveProxy();
			void						SetModified( bool inIsModified );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets a flag to indicate that the window is erased on update events.
										*	
										*	@param		inErase				Flag to indicate if the window is to be erased.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetEraseOnUpdate( bool inErase )						{ fEraseOnUpdate = inErase; }

										/*!
										*	Sets the minimum size a window can shrink to.
										*	
										*	@param		inMinSize			The minimum size a window can shrink to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetMinSize( const CP_Size& inMinSize )					{ fMinSize = inMinSize; }

										/*!
										*	Sets the maximum size a window can grow to.
										*	
										*	@param		inMaxSize			The maximum size a window can grow to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetMaxSize( const CP_Size& inMaxSize )					{ fMaxSize = inMaxSize; }

										/*!
										*	Sets the ideal size we want this window to be.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetIdealSize( const CP_Size& inIdealSize )				{ fIdealSize = inIdealSize; }
	//@}

	//! @name Title
	//@{
			void						SetTitle( const CP_String& inTitle );
			CP_String					GetTitle() const;
	//@}

	//! @name Testers
	//@{
			bool						IsComposited() const;
			bool						EraseOnUpdate() const;

	virtual	bool						IsVisible() const;
	//@}
	
	//! @name WindowTypeBase
	//@{
										/*!
										*	Returns the CP_WindowTypeBase object that controls the appearance and behavior
										*	of the window.
										*
										*	@exception	none
										*
										*	@return		CP_WindowTypeBase	The CP_WindowTypeBase that controls the windows type
										*									and appearance.
										*/
			CP_WindowTypeBase			GetWindowTypeBase()										{ return( fWindowTypeBase ); }

										/*!
										*	Sets the CP_WindowTypeBase object that controls the appearance and behavior
										*	of the window.
										*
										*	@param		inBase				The CP_WindowTypeBase that controls the type and appearance
										*									of the window.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetWindowTypeBase( const CP_WindowTypeBase& inBase )	{ fWindowTypeBase = inBase; }
	//@}

protected:
	//! @name AppleEvents
	//@{
	virtual	DescType					GetAppleClass() const;
	virtual	void						MakeSelfSpecifier(
												AEDesc& inContainerSpec,
												AEDesc& inOutSelfSpec );
	//@}

	//! @name AppleEventHandlers
	//@{
	virtual	void						DoAE_Close(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
			void						HandleBoundsChangedEvent( CP_Event_Base& inEvent );
	virtual	void						HandleDrawContentEvent( CP_Event_Base& inEvent );
	virtual	void						HandleWindowUpdateEvent( CP_Event_Base& inEvent );

			void						HandleCloseEvent( CP_Event_Base& inEvent );
			void						HandleClosedEvent( CP_Event_Base& inEvent );

	virtual void						OnClose();

			void						HandleActivatedEvent( CP_Event_Base& inEvent );
			void						HandleDeactivatedEvent( CP_Event_Base& inEvent );

			void						HandleMinSizeEvent( CP_Event_Base& inEvent );
			void						HandleMaxSizeEvent( CP_Event_Base& inEvent );

			void						HandleDragCompletedEvent( CP_Event_Base& inEvent );

	virtual	void						CommandUpdateStatus( CP_Event_Base& inEvent );
	virtual	void						CommandProcess( CP_Event_Base& inEvent );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the MacOS WindowRef that this window object is using for its platform window.
										*
										*	@param		inWindow			The OS window to use.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetImpPlatformWindow( OSWindow inWindow )			{ fOSWindow = inWindow; }

	virtual	void						SetLocation( const CP_Point& inLocation );
	virtual	void						SetSize( const CP_Size& inSize );
	virtual	void						SetPortBounds( const CP_Rect& inBounds );
	//@}

private:
	//! @name AppleEvents
	//@{
			void						SendAE_SetBounds(
												const CP_Rect& inBounds,
												bool inDoExecute );
	//@}
			
	//! @name Events
	//@{
											// Mouse
			void						HandleMouseMovedEvent( CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	CP_DataValueSignal					fWindowClosingSignal;	/*! Signal sent when the window is closing. */

	CP_WindowTypeBase					fWindowTypeBase;		/*! Object that controls the appearance and behavior of the platforms OS window. */
	
	CP_Size								fMinSize;				/*! Minimum size a window can shrink to. */
	CP_Size								fMaxSize;				/*! Maximum size a window can grow to. */
	CP_Size								fIdealSize;				/*! Ideal size for a window that does not change size. */
	
	SInt32								fWindowData;
	
	CP_Region							fUpdateRegion;
	
private:
	OSWindow							fOSWindow;				/*! Platforms window reference of the window. */
	bool								fEraseOnUpdate;			/*! Flag to indicate if we want to erase the window on updates. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_API_MAC_CARBON

#endif	// _H_CP_Window_Imp_CE

