//=========================================================================
//
//	FILE:				CP_ImageWell.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms image well control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ImageWell
#define _H_CP_ImageWell
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"
#include "CP_Structure_Defines.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ImageWell
*	@brief		Implementation of a platforms image well control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_ImageWell : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_ImageWell();
								CP_ImageWell(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										const CP_S_ViewContentInfo& inContentInfo );
	//! Destructor.
	virtual						~CP_ImageWell() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist			*PersistCreator();
	//@}
			
protected:
	//! @name Persistance
	//@{
	virtual	void				OnInitPersist(
										CP_View *inSuperview,
										CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:
	CP_S_ViewContentInfo		fContentInfo;			/*! Structure used to hold information about
															the contents of the button. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_WIN32
	//! @name Drawing
	//@{
	virtual	void				OnDraw(
										CP_Canvas& inCanvas,
										CP_Region& inUpdateArea );
	//@}

private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_LINUX
	//! @name Drawing
	//@{
	virtual	void				OnDraw(
										CP_Canvas& inCanvas,
										CP_Region& inUpdateArea );
	//@}

private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ImageWell
