//=========================================================================
//
//	FILE:				CP_ScrollBar.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms Scroll Bar control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ScrollBar
#define _H_CP_ScrollBar
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ScrollBar
*	@brief		Implementation of a platforms Scroll Bar control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_ScrollBar : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ScrollBar();
										CP_ScrollBar(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												bool inLiveTracking = true );
	//! Destructor.
	virtual								~CP_ScrollBar() throw();
	//@}
			
	//! @name Scroll Amount
	//@{
										/*!
										*	Returns the distance to scroll when the scroll bars arrows are clicked,
										*	when the user wants to scroll by a unit. This also corresponds to the CP_ScrollableViews
										*	scroll unit.
										*	
										*	@exception	none
										*	
										*	@return		SInt32			The distance to scroll.
										*/
			SInt32						GetLineSize()										{ return( fLineSize ); }

										/*!
										*	Sets the distance to scroll when the scroll bars arrows are clicked,
										*	when the user wants to scroll by a unit. This also corresponds to the CP_ScrollableViews
										*	scroll unit.
										*	
										*	@param		inLineSize		Amount to scroll when the arrows are clicked.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetLineSize( SInt32 inLineSize )					{ fLineSize = inLineSize; }

			SInt32						GetPageSize() const;

										/*!
										*	Sets the distance to scroll when the user requests to scroll by a page.
										*
										*	@param		inPageSize		The distance to scroll by page.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetPageSize( SInt32 inPageSize )					{ fPageSize = inPageSize; }
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist						*PersistCreator();
	//@}
			
	//! @name Value
	//@{
	virtual	SInt32						GetValue();
	virtual	void						SetValue( SInt32 inValue );
			
	virtual	SInt32						GetMin();
	virtual	void						SetMin( SInt32 inValue );

	virtual	SInt32						GetMax();
	virtual	void						SetMax( SInt32 inValue );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}
			
	// class data
protected:
	SInt32								fLineSize;		/*! Distance scrolled when the arrow is clicked. */
	SInt32								fPageSize;		/*! Distance scrolled when the page area is clicked. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Scrolling
	//@{
	virtual	void						DoScroll_Mac( SInt16 inPart );
	//@}

private:
	//! @name Scrolling
	//@{
	static pascal void					ScrollBarAction(
												ControlHandle inPlatformControl,
												SInt16 inPart );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												bool inLiveTracking = true );
	//@}	
#endif

#if TARGET_OS_WIN32
public:
			void						SetScrollMsgEventHandler(
												CPLAT::CP_EventHandler& inEventHandler,
												SInt16 inMessage );

private:
	//! @name Events
	//@{
			void						HandleHScrollEvent( CP_Event_Base& inEvent );
			void						HandleVScrollEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												bool inLiveTracking = true );
	//@}	
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												bool inLiveTracking = true );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ScrollBar
