//=========================================================================
//
//	FILE:				CP_Scroller.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A CP_View subclass that provides support for scrolling
//						of a view. The scroller has a CP_ScrollableView and 2
//						scroll bars to scroll the View.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 263 $
//						Latest Checkout:  $Date: 2005-08-28 13:10:16 -0700 (Sun, 28 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Scroller
#define _H_CP_Scroller
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_ScrollableView.h"
#include "CP_ScrollBar.h"
#include "CP_View.h"
#include "CP_View_Imp_ControlScroller.h"
#include "CP_View_Imp_GtkScroller.h"
#include "CP_View_Imp_HIScroller.h"
#include "CP_View_Imp_WinScroller.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Scroller
*	@brief		A CP_View subclass that provides support for scrolling
*				of a view. The scroller has a CP_ScrollableView and 2 scroll bars
*				to scroll the View.
*
*	@ingroup	UI
*/

class CP_EXPORT CP_Scroller : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Scroller();
										CP_Scroller(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inSmallScrollBars = false,
												bool inHasHorizontalScrollBar = true,
												bool inHasVerticalScrollBar = true,
												bool inHasGrowBoxSpace = true );
	//! Destructor.
	virtual								~CP_Scroller() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Frame
	//@{
	virtual	void						HandleBoundsChanged(
												const CP_Rect& inPreviousBounds,
												const CP_Rect& inNewBounds );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the CP_ScrollableView that this class is scrolling.
										*	
										*	@exception	none
										*	
										*	@return		CP_ScrollableView *		Pointer to the View that is being scrolled.
										*/
			CP_ScrollableView			*GetScrollingView()												{ return( fScrollingView ); }

										/*!
										*	Returns the size of the scroll bar.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		Size of the scroll bar, either 16 for normal, and 12 for small.
										*/
			SInt16						GetScrollBarSize()												{ return( fScrollBarSize ); }

	virtual	CP_Rect						GetContentBounds();
	//@}
			
	//! @name Setters
	//@{
			void						SetScrollingView( CP_ScrollableView *inScrollableView );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}
	// class data
protected:
	CP_ScrollableView					*fScrollingView;			/*! The View that is going to be scrolled. */
	SInt16								fScrollBarSize;				/*! Size of the scroll bars in pixels for either normal. */
	bool 								fHasHorizontalScrollBar;	/*! Flag to indicate if we have a horizontal scroll bar. */
	bool 								fHasVerticalScrollBar;		/*! Flag to indicate if we have a vertical scroll bar. */
	bool 								fHasGrowBoxSpace;			/*! Flag to indicate if we leave space for a grow box. */

	#if TARGET_OS_MAC
		#if TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020
			CP_View_Imp_HIScroller		*fScrollerImp;
			CP_View_Imp_ControlScroller	*fControlScrollerImp;
		#else
			CP_View_Imp_ControlScroller	*fScrollerImp;
		#endif
	#elif TARGET_OS_WIN32
		CP_View_Imp_WinScroller			*fScrollerImp;
	#elif TARGET_OS_LINUX
		CP_View_Imp_GtkScroller			*fScrollerImp;
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScrollBar = true,
												bool inHasVerticalScrollBar = true,
												bool inHasGrowBoxSpace = true );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScrollBar = true,
												bool inHasVerticalScrollBar = true,
												bool inHasGrowBoxSpace = true );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScrollBar = true,
												bool inHasVerticalScrollBar = true,
												bool inHasGrowBoxSpace = true );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Scroller

