//=========================================================================
//
//	FILE:				CP_TabView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms tab control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_TabView
#define _H_CP_TabView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"
#include "CP_Structure_Defines.h"

CPLAT_Begin_Namespace_CPLAT

class CP_TabInfo
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_TabInfo();
								CP_TabInfo( const CP_TabInfo& inTabInfo );
	//@}

	//! @name operators
	//@{
	const CP_TabInfo&			operator = ( const CP_TabInfo& nTabInfo );
	bool						operator == ( const CP_TabInfo& inTabInfo );
	bool						operator != ( const CP_TabInfo& inTabInfo )				{ return( ! (*this == inTabInfo) ); }
	//@}

	// class data
	CP_S_ViewContentInfo  		fContentType;		/*! Type of content in the tab, same as for buttons. */
	CP_String					fTabName;			/*! Text displayed on the tab. */
	bool						fEnabled;			/*! Flag to indicate if the tab is enabled. */
	SInt32						fViewID;			/*! CP_View that contains all of the views for this tab. */
	CP_View						*fView;				/*! The view being displayed for a tab. */
};

/*!	\fn typedef CP_CommandsArray
 *	std::vector that holds a list of CP_Commands. 
 */
typedef std::vector<CP_TabInfo, std::allocator<CP_TabInfo> >								CP_Tabs_Array;
typedef std::vector<CP_TabInfo, std::allocator<CP_TabInfo> >::iterator						CP_Tabs_Iterator;
typedef std::vector<CP_TabInfo, std::allocator<CP_TabInfo> >::reverse_iterator				CP_Tabs_ReverseIterator;

/*!
*	@class		CP_TabView
*	@brief		Implementation of a platforms tab control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_TabView : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TabView();
										CP_TabView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_TabView() throw();
	//@}
			
	//! @name Appearance
	//@{
			void						Show();
			void						Hide();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Tabs
	//@{
			void						AddTab( const CP_TabInfo& inTabInfo );
			void						RemoveTab( SInt32 inTabIndex );
			void						RemoveAllTabs();
			
			void						EnableTab(
												SInt32 inTab,
												bool inEnable );
			bool						IsTabEnabled( SInt16 inTab );
	//@}

	//! @name Value
	//@{
	virtual	void						SetValue( SInt32 inValue );
	//@}

protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	virtual	void						OnFinishPersist();
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Slots
	//@{
			void						HandleValueChangedEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Tabs
	//@{
			void						AddTabToControl( const CP_TabInfo& inTabInfo );
	//@}

	// class data
protected:
	CP_Tabs_Array						fTabsInfo;
	
	SInt32								fLastTabIndex;
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Drawing
	//@{
	virtual	void						SetForeBackColors();
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	virtual	CP_RGBColor					GetBackColor();

private:
	//! @name Events
	//@{
			void						HandleDrawItemEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TabView
